/********************************************************************************/
/*                                                                              */
/*              connect.cxx                                                     */
/*                                                                              */
/*      Message-based connection implementation using sockets                   */
/*                                                                              */
/********************************************************************************/


#include "connect_local.H"



/********************************************************************************/
/*                                                                              */
/*      ConnectInfo constructors/destructors                                    */
/*                                                                              */
/********************************************************************************/


ConnectInfo::ConnectInfo()
{
   conn_socket = new ConnectSocketInfo(this);
}



ConnectInfo::~ConnectInfo()
{
   conn_socket->close();

   delete conn_socket;
}



/********************************************************************************/
/*                                                                              */
/*      ConnectInfo X11 interface                                               */
/*                                                                              */
/********************************************************************************/


XtAppContext
ConnectInfo::setX11Context(XtAppContext ctx)
{
   return SimpleSocketInfo::useX11(ctx);
}



/********************************************************************************/
/*                                                                              */
/*      Methods to open a master socket                                         */
/*                                                                              */
/********************************************************************************/


int
ConnectInfo::openServer(const char * file)
{
   char host[1024];
   int port;

   if (!conn_socket->create()) return false;
   if (!conn_socket->bind()) return false;
   if (!conn_socket->listen()) return false;
   if (!conn_socket->getSockName(host,port)) return false;

   ofstream ofs(file);
   if (!ofs) return false;

   ofs << host << " " << port << endl;
   ofs.close();

   return true;
}



/********************************************************************************/
/*                                                                              */
/*      Methods to open a client socket                                         */
/*                                                                              */
/********************************************************************************/


int
ConnectInfo::openClient(const char * file)
{
   ifstream ifs(file);
   char host[1024];
   int port;

   if (!ifs) return false;

   ifs >> host >> port;
   if (ifs.fail()) return false;
   ifs.close();

   if (!conn_socket->create()) return false;
   if (!conn_socket->connect(host,port)) {
      conn_socket->close();
      return false;
    }

   return true;
}




/********************************************************************************/
/*                                                                              */
/*      Methods to open a server connection via accept                          */
/*                                                                              */
/********************************************************************************/


int
ConnectInfo::openAccept(Connect c)
{
   return conn_socket->accept(c->conn_socket);
}




/********************************************************************************/
/*                                                                              */
/*      Methods to close a socket                                               */
/*                                                                              */
/********************************************************************************/


void
ConnectInfo::close()
{
   conn_socket->close();
}



/********************************************************************************/
/*                                                                              */
/*      Methods to send a message                                               */
/*                                                                              */
/********************************************************************************/


void
ConnectInfo::sendMessage(int len,const void * msg)
{
   ConnectMsgLen ln = len;

   conn_socket->send(&ln,sizeof(ConnectMsgLen));
   conn_socket->send(msg,len);
}



/********************************************************************************/
/*                                                                              */
/*      Default callback methods                                                */
/*                                                                              */
/********************************************************************************/


void
ConnectInfo::acceptCallback()                                           { }


void
ConnectInfo::closeCallback()                                            { }


void
ConnectInfo::messageCallback(int len,void * msg)                        { }



/********************************************************************************/
/*                                                                              */
/*      ConnectSocketInfo constructors/destructors                              */
/*                                                                              */
/********************************************************************************/


ConnectSocketInfo::ConnectSocketInfo(Connect c)
{
   for_connect = c;
}



ConnectSocketInfo::~ConnectSocketInfo()                                 { }



/********************************************************************************/
/*                                                                              */
/*      ConnectSocketInfo callback methods                                      */
/*                                                                              */
/********************************************************************************/


void
ConnectSocketInfo::onAccept()
{
   for_connect->acceptCallback();
}


void
ConnectSocketInfo::onClose()
{
   for_connect->closeCallback();

   for_connect->close();
}



void
ConnectSocketInfo::onReceive()
{
   ConnectMsgLen len;
   Integer i;
   Character buf[1024];

   i = receive(&len,sizeof(ConnectMsgLen));
   if (i <= 0) return;

   assert(i == sizeof(ConnectMsgLen));

   assert(i <= 1024);
   i = receive(buf,len);
   assert(i == len);

   for_connect->messageCallback(len,buf);
}



/* end of connect.cxx */
