/********************************************************************************/
/*                                                                              */
/*              simplesocket.cxx                                                */
/*                                                                              */
/*      Basic socket class                                                      */
/*                                                                              */
/********************************************************************************/


#include "socket_local.H"
#include <errno.h>
#include <sys/systeminfo.h>


/********************************************************************************/
/*                                                                              */
/*      Local Storage                                                           */
/*                                                                              */
/********************************************************************************/


static  SimpleSocketX11         x11_info = NULL;

        Error                   SimpleSocketInfo::last_error = 0;



/********************************************************************************/
/*                                                                              */
/*      SimpleSocket constructors/destructors                                   */
/*                                                                              */
/********************************************************************************/


SimpleSocketInfo::SimpleSocketInfo()
{
   socket_id = -1;
   last_error = 0;
}


SimpleSocketInfo::~SimpleSocketInfo()
{ }



/********************************************************************************/
/*                                                                              */
/*      SimpleSocket creation methods                                           */
/*                                                                              */
/********************************************************************************/


bool
SimpleSocketInfo::create()
{
   socket_id = socket(AF_INET,SOCK_STREAM,0);

   if (x11_info == NULL) x11_info = new SimpleSocketX11Info;
   x11_info->remove(this);
   x11_info->addRead(socket_id,this);

   return socket_id >= 0;
}



void
SimpleSocketInfo::close()
{
   if (x11_info != NULL) x11_info->remove(this);

   ::close(socket_id);
   socket_id = -1;
}



/********************************************************************************/
/*                                                                              */
/*      SimpleSocket binding methods                                            */
/*                                                                              */
/********************************************************************************/


bool
SimpleSocketInfo::bind(int port)
{
   sockaddr_in name;
   int nln = sizeof(name);

   memset(&name,0,nln);
   name.sin_family = AF_INET;
   name.sin_port = port;

   if (::bind(socket_id,(struct sockaddr *) &name,nln) < 0) {
      last_error = errno;
      return false;
    }

   return true;
}



bool
SimpleSocketInfo::getSockName(char * host,int& port)
{
   sockaddr_in name;
   int nln = sizeof(name);
   Character hbuf[1024];
   hostent * he;

   if (getsockname(socket_id,(struct sockaddr *)&name,&nln) < 0) {
      last_error = errno;
      return false;
    }

   if (name.sin_addr.s_addr == 0) {
      sysinfo(SI_HOSTNAME,hbuf,1024);
      strcpy(host,hbuf);
      he = gethostbyname(hbuf);
      name.sin_addr.s_addr = * ((int *) he->h_addr);
    }
   else {
      he = gethostbyaddr((char *) &name,nln,AF_INET);
      strcpy(host,he->h_name);
    }

   port = name.sin_port;

   return true;
}



bool
SimpleSocketInfo::enableNonblocking(bool fg)
{
   if (socket_id < 0) return false;

   int i = fcntl(socket_id,F_GETFL,0);
   int j = 0;

#ifdef O_NONBLOCK
   j |= O_NONBLOCK;
#endif
#ifdef O_NDELAY
   j |= O_NDELAY;
#endif

   if (fg) i |= j;
   else i &= ~j;

   if (fcntl(socket_id,F_SETFL,i) < 0) {
      last_error = errno;
      return false;
    }

   return true;
}


bool
SimpleSocketInfo::listen(int n)
{
   if (::listen(socket_id,n) < 0) {
      last_error = errno;
      return false;
    }

   if (x11_info == NULL) x11_info = new SimpleSocketX11Info;
   x11_info->remove(this);                      // remove read here
   x11_info->addAccept(socket_id,this);

   return true;
}



bool
SimpleSocketInfo::accept(SimpleSocket news)
{
   sockaddr addr;
   int addrlen = sizeof(addr);
   int ns;

   if (news == NULL) return FALSE;

   ns = ::accept(socket_id,&addr,&addrlen);

   if (ns < 0) {
      last_error = errno;
      return false;
    }

   news->socket_id = ns;
   if (x11_info == NULL) x11_info = new SimpleSocketX11Info;
   x11_info->addRead(ns,news);

   return true;
}



bool
SimpleSocketInfo::connect(char * host,int port)
{
   sockaddr_in name;
   char fbuf[128];

   memset(&name,0,sizeof(name));
   name.sin_family = AF_INET;
   name.sin_port = port;
   sysinfo(SI_HOSTNAME,fbuf,128);
   if (strcmp(fbuf,host) == 0) name.sin_addr.s_addr = htonl(INADDR_LOOPBACK);
   else {
      hostent * he = gethostbyname(host);
      if (he == NULL) return false;
      else name.sin_addr.s_addr = *((int *) he->h_addr);
    }

   while (::connect(socket_id,(sockaddr *) &name,sizeof(name)) < 0) {
      if (errno != EINPROGRESS) {
         last_error = errno;
         return false;
       }
    }

   onConnect();

   return true;
}



/********************************************************************************/
/*                                                                              */
/*      SimpleSocket send and receive methods                                   */
/*                                                                              */
/********************************************************************************/


int
SimpleSocketInfo::send(const void * buf,int bln)
{
   int i = ::send(socket_id,(const char *) buf,bln,0);

   if (i < 0) last_error = errno;
   else if (i < bln) x11_info->addWrite(socket_id,this);

   return i;
}



int
SimpleSocketInfo::receive(void * buf,int bln)
{
   int i = ::recv(socket_id,(char *) buf,bln,0);

   if (i < 0) last_error = errno;

   return i;
}



/********************************************************************************/
/*                                                                              */
/*      X11 interface                                                           */
/*                                                                              */
/********************************************************************************/


void
SimpleSocketInfo::waitForEvent()
{
   if (x11_info == NULL) x11_info = new SimpleSocketX11Info;
   x11_info->waitForEvent();
}


XtAppContext
SimpleSocketInfo::useX11(XtAppContext ctx)
{
   if (ctx != NULL || x11_info == NULL) x11_info = new SimpleSocketX11Info(ctx);

   return x11_info->context();
}



/********************************************************************************/
/*                                                                              */
/*      SimpleSocketInfo default handlers                                       */
/*                                                                              */
/********************************************************************************/


void
SimpleSocketInfo::onAccept()                                  { }

void
SimpleSocketInfo::onClose()                                   { }

void
SimpleSocketInfo::onConnect()                                 { }

void
SimpleSocketInfo::onReceive()                                 { }

void
SimpleSocketInfo::onSend()                                    { }




/********************************************************************************/
/*                                                                              */
/*      SimpleSocketX11Info constructors/destructors                            */
/*                                                                              */
/********************************************************************************/


SimpleSocketX11Info::SimpleSocketX11Info(XtAppContext ctx)
{
   if (ctx == NULL) x11_context = XtCreateApplicationContext();
   else x11_context = ctx;
}



SimpleSocketX11Info::~SimpleSocketX11Info()                     { }




/********************************************************************************/
/*                                                                              */
/*      SimpleSocketX11Info methods for waiting for an event                    */
/*                                                                              */
/********************************************************************************/


static void
msg_wait_timeout(void * ud,unsigned long * tp)                          { }



void
SimpleSocketX11Info::waitForEvent(int pending)
{
   if (x11_info == NULL) x11_info = new SimpleSocketX11Info;

   Integer which = XtIMAlternateInput|XtIMTimer;

   if (pending < 0) {
      while (XtAppPending(x11_context)) {
         XtAppProcessEvent(x11_context,XtIMAll);
       }
      return;
    }

   if (pending > 0) {
      XtAppAddTimeOut(x11_context,2000,msg_wait_timeout,NULL);
    }

   XtAppProcessEvent(x11_context,which);
}




/********************************************************************************/
/*                                                                              */
/*      SimpleSocketX11Info methods for adding callbacks                        */
/*                                                                              */
/********************************************************************************/


void
SimpleSocketX11Info::addAccept(int fd,SimpleSocket s)
{
   XtInputId id = XtAppAddInput(x11_context,fd,(XtPointer) 1,handleAccept,s);

   callback_list.push_back(SimpleSocketX11Callback(id,s,fd));
}



void
SimpleSocketX11Info::addRead(int fd,SimpleSocket s)
{
   XtInputId id = XtAppAddInput(x11_context,fd,(XtPointer) 1,handleRead,s);

   callback_list.push_back(SimpleSocketX11Callback(id,s,fd));
}



void
SimpleSocketX11Info::addWrite(int fd,SimpleSocket s)
{
   XtInputId id = XtAppAddInput(x11_context,fd,(XtPointer) 2,handleWrite,s);

   callback_list.push_back(SimpleSocketX11Callback(id,s,fd));
}



/********************************************************************************/
/*                                                                              */
/*      SimpleSocketX11Info methods for removing callbacks                      */
/*                                                                              */
/********************************************************************************/


void
SimpleSocketX11Info::remove(SimpleSocket s,XtInputId id)
{
   SimpleSocketRemovePred pred(s,id);
   SimpleSocketX11CallbackListIter nend = remove_if(callback_list.begin(),callback_list.end(),pred);

   callback_list.erase(nend,callback_list.end());
}



SimpleSocketRemovePred::SimpleSocketRemovePred(SimpleSocket s,XtInputId id)
{
   for_socket = s;
   for_id = id;
}


int
SimpleSocketRemovePred::operator() (SimpleSocketX11Callback& cb)
{
   Bool fg = FALSE;

   if (for_socket == NULL || for_socket == cb.for_socket) {
      if (for_id == 0 || for_id == cb.x11_id) fg = TRUE;
    }

   if (fg && cb.x11_id != NULL) {
      XtRemoveInput(cb.x11_id);
      cb.x11_id = NULL;
    }

   return fg;
}



/********************************************************************************/
/*                                                                              */
/*      SimpleSocketX11Info methods for handling callbacks                      */
/*                                                                              */
/********************************************************************************/


void
SimpleSocketX11Info::handleAccept(void * sp,int *,XtInputId *)
{
   SimpleSocket s = (SimpleSocket) sp;

   s->onAccept();
}



void
SimpleSocketX11Info::handleRead(void * sp,int * fdp,XtInputId *)
{
   SimpleSocket s = (SimpleSocket) sp;
   Integer fd = *fdp;
   Character buf[10];

   int i = recv(fd,buf,1,MSG_PEEK);
   if (i > 0) s->onReceive();
   else if (i == 0) s->onClose();
}



void
SimpleSocketX11Info::handleWrite(void * sp,int *,XtInputId * xid)
{
   SimpleSocket s = (SimpleSocket) sp;

   s->onSend();

   x11_info->remove(s,*xid);
}



/* end of simplesocket.cxx */
