/********************************************************************************/
/*                                                                              */
/*              tchat_server.H                                                  */
/*                                                                              */
/*      Local definitions for the CHAT server using threads                     */
/*                                                                              */
/********************************************************************************/


#ifndef TCHAT_SERVER_ALREADY_DEFINED
#define TCHAT_SERVER_ALREADY_DEFINED

#include "chat.H"
#include "chat_message.H"
#include "simple_thread.H"

#include "stddef.H"
#include "connect.H"
#include <baum.H>



/********************************************************************************/
/*                                                                              */
/*      Type Definitions                                                        */
/*                                                                              */
/********************************************************************************/


typedef class TChatServerInfo *         TChatServer;
typedef class TChatConnectInfo *        TChatConnect;
typedef class TChatUserCallbackInfo *   TChatUserCallback;
typedef class TChatUserThreadInfo *     TChatUserThread;
typedef class TChatUserInfo *           TChatUser;
typedef class TChatUserConnectInfo *    TChatUserConnect;
typedef class TChatCommandInfo *        TChatCommand;

typedef list<TChatUser>                 TChatUserList;
typedef TChatUserList::iterator         TChatUserListIter;
typedef list<TChatCommand>              TChatCommandList;
typedef TChatCommandList::iterator      TChatCommandListIter;




/********************************************************************************/
/*                                                                              */
/*      TChatServerInfo -- main server for chattering                           */
/*                                                                              */
/********************************************************************************/


class TChatServerInfo {

private:
   BAUM x11_context;
   TChatConnect server_connect;
   TChatUserList active_list;
   SimpleMutex server_mutex;

public:
   TChatServerInfo(int,const char**);
   ~TChatServerInfo();

   void process();

public:
   void addUser(TChatUser);
   void removeUser(TChatUser);
   void getMessage(TChatUser,ConstText msg);

   Connect masterConnect() const;
};




/********************************************************************************/
/*                                                                              */
/*      TChatConnectInfo -- connection to the server                            */
/*                                                                              */
/********************************************************************************/


class TChatConnectInfo : public ConnectInfo {

private:
   TChatServer for_server;

public:
   TChatConnectInfo(TChatServer);
   ~TChatConnectInfo();

private:
   void acceptCallback();
};



/********************************************************************************/
/*                                                                              */
/*      TChatCommandInfo -- contents of a command                               */
/*                                                                              */
/********************************************************************************/


struct TChatCommandInfo {

public:
   enum Cmd { SET_USER, MSG_FROM, MSG_TO, USER_START, USER_END, EXIT };

private:
   Cmd cmd_type;
   StdString cmd_arg;

public:
   TChatCommandInfo(Cmd,ConstText = NULL);
   ~TChatCommandInfo();

   bool execute(TChatUser user);        // return false to exit
};



/********************************************************************************/
/*                                                                              */
/*      TChatUserThreadInfo -- thread for a single user                         */
/*                                                                              */
/********************************************************************************/


class TChatUserThreadInfo : public SimpleThread {

private:
   TChatUser for_user;

public:
   TChatUserThreadInfo(TChatUser usr);
   ~TChatUserThreadInfo();

   int run();
};




/********************************************************************************/
/*                                                                              */
/*      TChatUserInfo -- information about a suer                               */
/*                                                                              */
/********************************************************************************/


class TChatUserInfo : public ChatSessionInfo {

private:
   TChatServer for_server;
   TChatUserCallback user_callback;
   TChatUserConnect user_connect;
   TChatCommandList command_queue;
   StdString user_name;
   TChatUserThread user_thread;
   SimpleSemaphore queue_sema;
   SimpleMutex queue_mutex;

public:
   TChatUserInfo(TChatServer);
   ~TChatUserInfo();

   ConstText name() const                       { return user_name.c_str(); }
   TChatUserCallback callback() const           { return user_callback; }

   void run();
   void queueCommand(TChatCommand);

   void setUserName(const char * id);
   void message(const char * message);
};




/********************************************************************************/
/*                                                                              */
/*      TChatUserConnectInfo -- server connection to a user                     */
/*                                                                              */
/********************************************************************************/


class TChatUserConnectInfo : public ConnectInfo {

private:
   TChatUser for_user;

public:
   TChatUserConnectInfo(TChatUser,Connect);
   ~TChatUserConnectInfo();

private:
   void closeCallback();
   void messageCallback(int len,void * msg);
};



/********************************************************************************/
/*                                                                              */
/*      TChatUserCallbackInfo -- callback to user application                   */
/*                                                                              */
/********************************************************************************/


class TChatUserCallbackInfo : public ChatCallbackInfo {

private:
   TChatUserConnect user_connect;

public:
   TChatUserCallbackInfo(TChatUserConnect);
   ~TChatUserCallbackInfo();

   void chatMessage(const char * user,const char * message);
   void chatUserStart(const char * user);
   void chatUserEnd(const char * user);
};



#endif


/* end of tchat_server.H */














