/********************************************************************************/
/*                                                                              */
/*              tchatuser.cxx                                                   */
/*                                                                              */
/*      Implementation of a specific user for the chat server                   */
/*                                                                              */
/********************************************************************************/


#include "tchat_server.H"



/********************************************************************************/
/*                                                                              */
/*      TChatUserInfo constructor/destructors                                   */
/*                                                                              */
/********************************************************************************/


TChatUserInfo::TChatUserInfo(TChatServer cs)
{
   for_server = cs;
   user_connect = new TChatUserConnectInfo(this,for_server->masterConnect());
   user_name = "???";
   user_callback = new TChatUserCallbackInfo(user_connect);

   user_thread = new TChatUserThreadInfo(this);
   user_thread->createThread();
}



TChatUserInfo::~TChatUserInfo()
{
   for_server->removeUser(this);
   delete user_connect;
   delete user_callback;
   delete user_thread;
}



/********************************************************************************/
/*                                                                              */
/*      TChatUserInfo command queue and process methods                         */
/*                                                                              */
/********************************************************************************/


void
TChatUserInfo::run()
{
   TChatCommand cmd;

   for ( ; ; ) {
      queue_sema.lock();                        // wait for a message

      queue_mutex.lock();                       // to get item off queue
      cmd = command_queue.front();
      command_queue.pop_front();
      queue_mutex.unlock();

      if (!cmd->execute(this)) break;

      delete cmd;
    }

   for_server->removeUser(this);

   delete this;
}



void
TChatUserInfo::queueCommand(TChatCommand cmd)
{
   queue_mutex.lock();                          // to put item on queue
   command_queue.push_back(cmd);
   queue_mutex.unlock();

   queue_sema.unlock();                         // to allow command to be processed
}




/********************************************************************************/
/*                                                                              */
/*      TChatUserInfo action methods                                            */
/*                                                                              */
/********************************************************************************/


void
TChatUserInfo::setUserName(const char * id)
{
   user_name = id;
   for_server->addUser(this);
}



void
TChatUserInfo::message(const char * msg)
{
   for_server->getMessage(this,msg);
}



/********************************************************************************/
/*                                                                              */
/*      TChatUserConnectInfo constructors/destructors                           */
/*                                                                              */
/********************************************************************************/


TChatUserConnectInfo::TChatUserConnectInfo(TChatUser cu,Connect master)
{
   for_user = cu;
   master->openAccept(this);
}


TChatUserConnectInfo::~TChatUserConnectInfo()                             { }




/********************************************************************************/
/*                                                                              */
/*      TChatUserConnectInfo callback methods                                   */
/*                                                                              */
/********************************************************************************/


void
TChatUserConnectInfo::closeCallback()
{
   TChatCommand cmd = new TChatCommandInfo(TChatCommandInfo::EXIT);
   for_user->queueCommand(cmd);
}



void
TChatUserConnectInfo::messageCallback(int len,void * msg)
{
   Text tmsg = (Text) msg;
   ChatMsgType mtype = tmsg[0];
   Text mbody = &tmsg[2];
   TChatCommand cmd = NULL;

   switch (mtype) {
      case CHAT_MSG_SET_USER :
         cmd = new TChatCommandInfo(TChatCommandInfo::SET_USER,mbody);
         break;
      case CHAT_MSG_CLIENT :
         cmd = new TChatCommandInfo(TChatCommandInfo::MSG_FROM,mbody);
         break;
      default :
         break;
    }

   if (cmd != NULL) for_user->queueCommand(cmd);
}



/********************************************************************************/
/*                                                                              */
/*      TChatUserCallbackInfo constructors/destructors                          */
/*                                                                              */
/********************************************************************************/


TChatUserCallbackInfo::TChatUserCallbackInfo(TChatUserConnect conn)
{
   user_connect = conn;
}


TChatUserCallbackInfo::~TChatUserCallbackInfo()               { }




/********************************************************************************/
/*                                                                              */
/*      TChatUserCallbackInfo callback handlers                                 */
/*                                                                              */
/********************************************************************************/


void
TChatUserCallbackInfo::chatMessage(const char * u,const char * m)
{
   Character buf[10240];
   ostrstream ost(buf,10240);

   ost << CHAT_MSG_SERVER << " " << u << CHAT_MSG_SEPARATOR << m << ends;

   user_connect->sendMessage(buf);
}



void
TChatUserCallbackInfo::chatUserStart(const char * u)
{
   Character buf[1024];
   ostrstream ost(buf,1024);

   ost << CHAT_MSG_NEW_USER << " " << u << ends;

   user_connect->sendMessage(buf);
}



void
TChatUserCallbackInfo::chatUserEnd(const char * u)
{
   Character buf[1024];
   ostrstream ost(buf,1024);

   ost << CHAT_MSG_REMOVE_USER << " " << u << ends;

   user_connect->sendMessage(buf);
}




/********************************************************************************/
/*                                                                              */
/*      TChatCommandInfo constructors/destructors                               */
/*                                                                              */
/********************************************************************************/


TChatCommandInfo::TChatCommandInfo(Cmd c,ConstText a)
{
   cmd_type = c;
   cmd_arg = (a == NULL ? "" : a);
}



TChatCommandInfo::~TChatCommandInfo()                           { }




/********************************************************************************/
/*                                                                              */
/*      TChatCommandInfo methods                                                */
/*                                                                              */
/********************************************************************************/


bool
TChatCommandInfo::execute(TChatUser usr)
{
   bool fg = true;
   ConstText a = cmd_arg.c_str();

   switch (cmd_type) {
      case SET_USER :
         usr->setUserName(a);
         break;
      case MSG_FROM :
         usr->message(a);
         break;
      case MSG_TO :
         usr->callback()->chatMessage(usr->name(),a);
         break;
      case USER_START :
         usr->callback()->chatUserStart(usr->name());
         break;
      case USER_END :
         usr->callback()->chatUserEnd(usr->name());
         break;
      case EXIT :
         fg = false;
         break;
    }

   return fg;
}



/********************************************************************************/
/*                                                                              */
/*      TChatUserThreadInfo methods                                             */
/*                                                                              */
/********************************************************************************/


TChatUserThreadInfo::TChatUserThreadInfo(TChatUser u)
{
   for_user = u;
}



TChatUserThreadInfo::~TChatUserThreadInfo()                             { }



int
TChatUserThreadInfo::run()
{
   for_user->run();

   return 0;
}



/* end of tchatuser.cxx */
